@@ -37,32 +37,27 @@ gboolean info_tag_key_collision(tag_t *tags, tag_t *tag) {
   return FALSE;
 }
 
-static gboolean
-view_selection_func(GtkTreeSelection *selection, GtkTreeModel *model,
-		     GtkTreePath *path, gboolean path_currently_selected,
-		     gpointer userdata) {
-  tag_context_t *context = (tag_context_t*)userdata;
+static void changed(GtkTreeSelection *treeselection, gpointer user_data) {
+  GtkWidget *list = (GtkWidget*)user_data;
+
+  GtkTreeModel *model;
   GtkTreeIter iter;
+  gboolean selected = list_get_selected(list, &model, &iter);
 
-  if(gtk_tree_model_get_iter(model, &iter, path)) {
-    g_assert(gtk_tree_path_get_depth(path) == 1);
-
+  if(selected) {
     tag_t *tag;
     gtk_tree_model_get(model, &iter, TAG_COL_DATA, &tag, -1);
 
     /* you just cannot delete or edit the "created_by" tag */
-    if(!tag || strcasecmp(tag->key, "created_by") == 0) {
-      list_button_enable(context->list, LIST_BUTTON_REMOVE, FALSE);
-      list_button_enable(context->list, LIST_BUTTON_EDIT, FALSE);
-    } else {
-      list_button_enable(context->list, LIST_BUTTON_REMOVE, TRUE);
-      list_button_enable(context->list, LIST_BUTTON_EDIT, TRUE);
-    }
+    if(!tag || strcasecmp(tag->key, "created_by") == 0) 
+      selected = FALSE;
   }
   
-  return TRUE; /* allow selection state to change */
+  list_button_enable(GTK_WIDGET(list), LIST_BUTTON_REMOVE, selected);
+  list_button_enable(GTK_WIDGET(list), LIST_BUTTON_EDIT, selected);
 }
 
+
 static void update_collisions(GtkListStore *store, tag_t *tags) {
   GtkTreeIter iter;
   tag_t *tag = NULL;
@@ -109,10 +104,6 @@ static void on_tag_remove(GtkWidget *but, tag_context_t *context) {
 
     update_collisions(context->store, *context->tag);
   }
-  
-  /* disable remove and edit buttons */
-  list_button_enable(context->list, LIST_BUTTON_REMOVE, FALSE);
-  list_button_enable(context->list, LIST_BUTTON_EDIT, FALSE);
 }
 
 static gboolean tag_edit(tag_context_t *context) {
@@ -151,7 +142,7 @@ static gboolean tag_edit(tag_context_t *context) {
 		   0, 1, 0, 1, 0, 0, 0, 0);
   gtk_misc_set_alignment(GTK_MISC(label), 1.f, 0.5f);
   gtk_table_attach_defaults(GTK_TABLE(table), 
-			    key = gtk_entry_new(), 1, 2, 0, 1);
+			    key = entry_new(), 1, 2, 0, 1);
   gtk_entry_set_activates_default(GTK_ENTRY(key), TRUE);
   HILDON_ENTRY_NO_AUTOCAP(key);
 
@@ -159,7 +150,7 @@ static gboolean tag_edit(tag_context_t *context) {
 		   0, 1, 1, 2, 0, 0, 0, 0);
   gtk_misc_set_alignment(GTK_MISC(label), 1.f, 0.5f);
   gtk_table_attach_defaults(GTK_TABLE(table), 
-		    value = gtk_entry_new(), 1, 2, 1, 2);
+		    value = entry_new(), 1, 2, 1, 2);
   gtk_entry_set_activates_default(GTK_ENTRY(value), TRUE);
   HILDON_ENTRY_NO_AUTOCAP(value);
 
@@ -247,7 +238,7 @@ static void on_tag_add(GtkWidget *button, tag_context_t *context) {
 		     -1);
 
   gtk_tree_selection_select_iter(
-	 list_get_selection(context->list), &iter);
+     list_get_selection(context->list), &iter);
 
   if(!tag_edit(context)) {
     printf("cancelled\n");
@@ -273,22 +264,28 @@ void info_tags_replace(tag_context_t *context) {
 }
 
 static void on_relations(GtkWidget *button, tag_context_t *context) {
-  relation_add_dialog(context->dialog, context->appdata, &context->object);
+  relation_membership_dialog(context->dialog, context->appdata, 
+			     &context->object);
 }
 
 static GtkWidget *tag_widget(tag_context_t *context) {
-  context->list = list_new(LIST_HILDON_WITH_HEADERS_ON_MAEMO5);
+  context->list = list_new(LIST_HILDON_WITHOUT_HEADERS);
 
   list_set_static_buttons(context->list, 0, G_CALLBACK(on_tag_add), 
 	  G_CALLBACK(on_tag_edit), G_CALLBACK(on_tag_remove), context);
 
-  list_set_selection_function(context->list, view_selection_func, context);
+  list_override_changed_event(context->list, changed, context->list);
 
   list_set_user_buttons(context->list, 
 			LIST_BUTTON_USER0, _("Last"),      on_tag_last,
 			LIST_BUTTON_USER2, _("Relations"), on_relations,
 			0);
 
+  /* "relations of a relation" is something we can't handle correctly */
+  /* at the moment */
+  if(context->object.type == RELATION)
+    list_button_enable(context->list, LIST_BUTTON_USER2, FALSE);
+
   /* setup both columns */
   list_set_columns(context->list, 
       _("Key"),   TAG_COL_KEY,   
@@ -341,8 +338,12 @@ static void on_relation_members(GtkWidget *but, tag_context_t *context) {
   relation_show_members(context->dialog, context->object.relation);
 }
 
-static GtkWidget *details_widget(tag_context_t *context) {
-  GtkWidget *table = gtk_table_new(2, 2, FALSE);  // x, y
+static void table_attach(GtkWidget *table, GtkWidget *child, int x, int y) {
+  gtk_table_attach_defaults(GTK_TABLE(table), child, x, x+1, y, y+1);
+}
+
+static GtkWidget *details_widget(tag_context_t *context, gboolean big) {
+  GtkWidget *table = gtk_table_new(big?4:2, 2, FALSE);  // y, x
 
   user_t *user = OBJECT_USER(context->object);
   time_t stime = OBJECT_TIME(context->object);
@@ -350,18 +351,21 @@ static GtkWidget *details_widget(tag_context_t *context) {
 
   /* ------------ user ----------------- */
   if(user) {
+    if(big) table_attach(table, gtk_label_new(_("User:")), 0, 0);
+
     label = gtk_label_new(user->name);
     gtk_label_set_ellipsize(GTK_LABEL(label), PANGO_ELLIPSIZE_END);
-    gtk_table_attach_defaults(GTK_TABLE(table),  label, 0, 1, 0, 1);
+    table_attach(table, label, big?1:0, 0);
   }
 
   /* ------------ time ----------------- */
 
+  if(big) table_attach(table, gtk_label_new(_("Date/Time:")), 0, 1);
   struct tm *loctime = localtime(&stime);
   char time_str[32];
   strftime(time_str, sizeof(time_str), "%x %X", loctime);
   label = gtk_label_new(time_str);
-  gtk_table_attach_defaults(GTK_TABLE(table),  label, 1, 2, 0, 1);
+  table_attach(table, label, 1, big?1:0);
 
   /* ------------ coordinate (only for nodes) ----------------- */
   switch(context->object.type) {
@@ -369,17 +373,21 @@ static GtkWidget *details_widget(tag_context_t *context) {
     char pos_str[32];
     pos_lat_str(pos_str, sizeof(pos_str), context->object.node->pos.lat);
     label = gtk_label_new(pos_str);
-    gtk_table_attach_defaults(GTK_TABLE(table),  label, 0, 1, 1, 2);
+    if(big) table_attach(table, gtk_label_new(_("Latitude:")), 0, 2);
+    table_attach(table, label, big?1:0, big?2:1);
     pos_lat_str(pos_str, sizeof(pos_str), context->object.node->pos.lon);
     label = gtk_label_new(pos_str);
-    gtk_table_attach_defaults(GTK_TABLE(table),  label, 1, 2, 1, 2);
+    if(big) table_attach(table, gtk_label_new(_("Longitude:")), 0, 3);
+    table_attach(table, label, 1, big?3:1);
   } break;
 
   case WAY: {
-    char *nodes_str = g_strdup_printf(_("Length: %u nodes"), 
+    char *nodes_str = g_strdup_printf(_("%s%u nodes"), 
+	      big?"":_("Length: "),
 	      osm_way_number_of_nodes(context->object.way));
     label = gtk_label_new(nodes_str);
-    gtk_table_attach_defaults(GTK_TABLE(table),  label, 0, 1, 1, 2);
+    if(big) table_attach(table, gtk_label_new(_("Length:")), 0, 2);
+    table_attach(table, label, big?1:0, big?2:1);
     g_free(nodes_str);
     
     char *type_str = g_strdup_printf("%s (%s)",
@@ -390,7 +398,8 @@ static GtkWidget *details_widget(tag_context_t *context) {
 			       "area":"line");
  
     label = gtk_label_new(type_str);      
-    gtk_table_attach_defaults(GTK_TABLE(table),  label, 1, 2, 1, 2);
+    if(big) table_attach(table, gtk_label_new(_("Type:")), 0, 3);
+    table_attach(table, label, 1, big?3:1);
     g_free(type_str);
   } break;
 
@@ -424,10 +433,12 @@ static GtkWidget *details_widget(tag_context_t *context) {
       g_strdup_printf(_("Members: %d nodes, %d ways, %d relations"),
 		      nodes, ways, relations);
 
-    GtkWidget *member_btn = gtk_button_new_with_label(str);
+    GtkWidget *member_btn = button_new_with_label(str);
     gtk_signal_connect(GTK_OBJECT(member_btn), "clicked", 
 		       GTK_SIGNAL_FUNC(on_relation_members), context);
-    gtk_table_attach_defaults(GTK_TABLE(table), member_btn, 0, 2, 1, 2);
+
+    gtk_table_attach_defaults(GTK_TABLE(table), member_btn, 0, 2, 
+			      big?2:1, big?4:2);
 
     g_free(str);
     break;
@@ -455,7 +466,7 @@ static void info_more(tag_context_t *context) {
 				  GTK_RESPONSE_CANCEL);
 
   gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), 
-		     details_widget(context),
+		     details_widget(context, TRUE),
 		     FALSE, FALSE, 0);
   gtk_widget_show_all(dialog);
   gtk_dialog_run(GTK_DIALOG(dialog));
@@ -538,7 +549,7 @@ gboolean info_dialog(GtkWidget *parent, appdata_t *appdata, object_t *object) {
 #ifndef FREMANTLE
   /* -------- details box --------- */
   gtk_box_pack_start(GTK_BOX(GTK_DIALOG(context->dialog)->vbox), 
-		     details_widget(context),
+		     details_widget(context, FALSE),
 		     FALSE, FALSE, 0);
 #endif
 
