@@ -24,13 +24,12 @@
 typedef struct {
   object_t *item;
   appdata_t *appdata;
-  GtkWidget *dialog, *list;
+  GtkWidget *dialog, *view;
   GtkListStore *store;
 } relitem_context_t;
 
 enum {
-  RELITEM_COL_SELECTED = 0,
-  RELITEM_COL_TYPE,
+  RELITEM_COL_TYPE = 0,
   RELITEM_COL_ROLE,
   RELITEM_COL_NAME,
   RELITEM_COL_DATA,
@@ -99,21 +98,25 @@ static gboolean relation_add_item(GtkWidget *parent,
 				gtk_label_new(name));
 
   GtkWidget *hbox = gtk_hbox_new(FALSE, 8);
-  gtk_box_pack_start_defaults(GTK_BOX(hbox), gtk_label_new(_("Role:")));
+
+#ifdef FREMANTLE
+  if(!chain) 
+#endif
+    gtk_box_pack_start_defaults(GTK_BOX(hbox), gtk_label_new(_("Role:")));
 
   GtkWidget *entry = NULL;
   if(chain) {
-    entry = gtk_combo_box_entry_new_text();
+    entry = combo_box_entry_new(_("Role"));
 
     /* fill combo box with presets */
     while(chain) {
       role_chain_t *next = chain->next;
-      gtk_combo_box_append_text(GTK_COMBO_BOX(entry), chain->role);
+      combo_box_append_text(entry, chain->role);
       g_free(chain);
       chain = next;
     }
   } else
-    entry = gtk_entry_new();
+    entry = entry_new();
 
   gtk_box_pack_start_defaults(GTK_BOX(hbox), entry);
   gtk_box_pack_start_defaults(GTK_BOX(GTK_DIALOG(dialog)->vbox), hbox);
@@ -128,12 +131,12 @@ static gboolean relation_add_item(GtkWidget *parent,
   printf("user clicked ok\n");
 
   /* get role from dialog */
-  char *ptr = NULL;
+  const char *ptr = NULL;
 
-  if(GTK_IS_COMBO_BOX(entry))
-    ptr = gtk_combo_box_get_active_text(GTK_COMBO_BOX(entry));
+  if(GTK_WIDGET_TYPE(entry) == combo_box_entry_type()) 
+    ptr = combo_box_get_active_text(entry);
   else
-    ptr = (char*)gtk_entry_get_text(GTK_ENTRY(entry));
+    ptr = gtk_entry_get_text(GTK_ENTRY(entry));
 
   char *role = NULL;
   if(ptr && strlen(ptr)) role = g_strdup(ptr);
@@ -184,13 +187,6 @@ static void relation_remove_item(relation_t *relation, object_t *object) {
   g_assert(0);
 }
 
-static void relation_item_list_selected(relitem_context_t *context, 
-				   gboolean selected) {
-
-  list_button_enable(context->list, LIST_BUTTON_REMOVE, selected);
-  list_button_enable(context->list, LIST_BUTTON_EDIT, selected);
-}
-
 /* try to find something descriptive */
 static char *relation_get_descriptive_name(relation_t *relation) {
   char *name = osm_tag_get_by_key(OSM_TAG(relation), "ref");
@@ -200,6 +196,9 @@ static char *relation_get_descriptive_name(relation_t *relation) {
     name = osm_tag_get_by_key(OSM_TAG(relation), "note");
   if (!name)
     name = osm_tag_get_by_key(OSM_TAG(relation), "fix" "me");
+  if(!name)
+    name = g_strdup_printf("<ID #"ITEM_ID_FORMAT">", OSM_ID(relation));
+
   return name;
 }
 
@@ -211,112 +210,6 @@ static gboolean relation_info_dialog(GtkWidget *parent, appdata_t *appdata,
   return info_dialog(parent, appdata, &object);
 } 
 
-static void on_relation_item_add(GtkWidget *but, relitem_context_t *context) {
-  /* create a new relation */
-
-  relation_t *relation = osm_relation_new();
-  if(!relation_info_dialog(context->dialog, context->appdata, relation)) {
-    printf("tag edit cancelled\n");
-    osm_relation_free(relation);
-  } else {
-    osm_relation_attach(context->appdata->osm, relation);
-
-    /* add to list */
-
-    /* append a row for the new data */
-    char *name = relation_get_descriptive_name(relation);
-
-    GtkTreeIter iter;
-    gtk_list_store_append(context->store, &iter);
-    gtk_list_store_set(context->store, &iter,
-		       RELITEM_COL_SELECTED, FALSE,
-		       RELITEM_COL_TYPE, 
-		       osm_tag_get_by_key(OSM_TAG(relation), "type"),
-		       RELITEM_COL_NAME, name,
-		       RELITEM_COL_DATA, relation,
-		       -1);
-
-    gtk_tree_selection_select_iter(list_get_selection(context->list), &iter);
-  }
-}
-
-static relation_t *get_selection(relitem_context_t *context) {
-  GtkTreeSelection *selection;
-  GtkTreeModel     *model;
-  GtkTreeIter       iter;
-
-  selection = list_get_selection(context->list);
-  if(gtk_tree_selection_get_selected(selection, &model, &iter)) {
-    relation_t *relation;
-    gtk_tree_model_get(model, &iter, RELITEM_COL_DATA, &relation, -1);
-    return(relation);
-  }
-  return NULL;
-}
-
-static void on_relation_item_edit(GtkWidget *but, relitem_context_t *context) {
-  relation_t *sel = get_selection(context);
-  if(!sel) return;
-
-  printf("edit relation item #" ITEM_ID_FORMAT "\n", OSM_ID(sel));
-
-  if (!relation_info_dialog(context->dialog, context->appdata, sel))
-    return;
-
-  // Locate the changed item
-  GtkTreeIter iter;
-  gboolean valid = gtk_tree_model_get_iter_first(
-    GTK_TREE_MODEL(context->store), &iter);
-  while (valid) {
-    relation_t *row_rel;
-    gtk_tree_model_get(GTK_TREE_MODEL(context->store), &iter,
-                       RELITEM_COL_DATA, &row_rel,
-                       -1);
-    if (row_rel == sel)
-      break;
-    valid = gtk_tree_model_iter_next(GTK_TREE_MODEL(context->store), &iter);
-  }
-  if (!valid)
-    return;
-
-  // Found it. Update all visible fields that belong to the relation iself.
-  gtk_list_store_set(context->store, &iter,
-     RELITEM_COL_TYPE,    osm_tag_get_by_key(OSM_TAG(sel), "type"),
-     RELITEM_COL_NAME,    relation_get_descriptive_name(sel),
-    -1);
-
-  // Order will probably have changed, so refocus
-  list_focus_on(context->list, &iter, TRUE);
-}
-
-/* remove the selected relation */
-static void on_relation_item_remove(GtkWidget *but, relitem_context_t *context) {
-  relation_t *sel = get_selection(context);
-  if(!sel) return;
-
-  printf("remove relation #" ITEM_ID_FORMAT "\n", OSM_ID(sel));
-
-  gint members = osm_relation_members_num(sel);
-
-  if(members) 
-    if(!yes_no_f(context->dialog, NULL, 0, 0,
-		 _("Delete non-empty relation?"), 
-		 _("This relation still has %d members. "
-		   "Delete it anyway?"), members))
-      return;
-  
-  /* first remove selected row from list */
-  GtkTreeIter       iter;
-  GtkTreeSelection *selection = list_get_selection(context->list);
-  if(gtk_tree_selection_get_selected(selection, NULL, &iter)) 
-    gtk_list_store_remove(context->store, &iter);
-
-  /* then really delete it */
-  osm_relation_delete(context->appdata->osm, sel, FALSE); 
-
-  relation_item_list_selected(context, FALSE);
-}
-
 static char *relitem_get_role_in_relation(object_t *item, relation_t *relation) {
   member_t *member = relation->member;
   while(member) {
@@ -340,45 +233,6 @@ static char *relitem_get_role_in_relation(object_t *item, relation_t *relation)
   return NULL;
 }
 
-static void
-relitem_toggled(GtkCellRendererToggle *cell, const gchar *path_str,
-		relitem_context_t *context) {
-  GtkTreePath *path;
-  GtkTreeIter iter;
-
-  path = gtk_tree_path_new_from_string(path_str);
-  gtk_tree_model_get_iter(GTK_TREE_MODEL(context->store), &iter, path);
-  gtk_tree_path_free(path);
-
-  /* get current enabled flag */
-  gboolean enabled;
-  relation_t *relation = NULL;
-  gtk_tree_model_get(GTK_TREE_MODEL(context->store), &iter, 
-		     RELITEM_COL_SELECTED, &enabled, 
-		     RELITEM_COL_DATA, &relation, 
-		     -1);
-
-  list_pre_inplace_edit_tweak(GTK_TREE_MODEL(context->store));
-
-  if(!enabled) {
-    printf("will now become be part of this relation\n");
-    if(relation_add_item(context->dialog, relation, context->item))
-      gtk_list_store_set(context->store, &iter, 
-		 RELITEM_COL_SELECTED, TRUE, 
-		 RELITEM_COL_ROLE, 
-		 relitem_get_role_in_relation(context->item, relation),
-		 -1);
-  } else {
-    printf("item will not be part of this relation anymore\n");
-    relation_remove_item(relation, context->item);
-    gtk_list_store_set(context->store, &iter, 
-		       RELITEM_COL_SELECTED, FALSE, 
-		       RELITEM_COL_ROLE, NULL, 
-		       -1);
-  }
-
-}
-
 static gboolean relitem_is_in_relation(object_t *item, relation_t *relation) {
   member_t *member = relation->member;
   while(member) {
@@ -402,29 +256,141 @@ static gboolean relitem_is_in_relation(object_t *item, relation_t *relation) {
   return FALSE;
 }
 
-static GtkWidget *relation_item_list_widget(relitem_context_t *context) {
-  context->list = list_new(LIST_HILDON_WITH_HEADERS);
+static void changed(GtkTreeSelection *sel, gpointer user_data) {
+  relitem_context_t *context = (relitem_context_t*)user_data;
 
-  list_set_columns(context->list, 
-		   _(""), RELITEM_COL_SELECTED, LIST_FLAG_TOGGLE, 
-		            G_CALLBACK(relitem_toggled), context,
-		   _("Type"), RELITEM_COL_TYPE, 0,
-		   _("Role"), RELITEM_COL_ROLE, 0,
-		   _("Name"), RELITEM_COL_NAME, LIST_FLAG_ELLIPSIZE,
-		   NULL);
+  printf("relation-edit changed event\n");  
+
+  /* we need to know what changed in order to let the user acknowlege it! */
+
+  /* walk the entire store */
+
+  GtkTreeIter iter;
+  gboolean done = FALSE, ok = 
+    gtk_tree_model_get_iter_first(GTK_TREE_MODEL(context->store), &iter);
+  while(ok && !done) {
+    relation_t *relation = NULL;
+    gtk_tree_model_get(GTK_TREE_MODEL(context->store), &iter, 
+		       RELITEM_COL_DATA, &relation, -1);    
+    g_assert(relation);
+
+    if(!relitem_is_in_relation(context->item, relation) && 
+       gtk_tree_selection_iter_is_selected(sel, &iter)) {
+
+      printf("selected: "ITEM_ID_FORMAT"\n", OSM_ID(relation));
+
+      /* either accept this or unselect again */
+      if(relation_add_item(context->dialog, relation, context->item))
+	gtk_list_store_set(context->store, &iter, 
+	   RELITEM_COL_ROLE, 
+	   relitem_get_role_in_relation(context->item, relation),
+			   -1);
+      else
+	gtk_tree_selection_unselect_iter(sel, &iter);
+
+      done = TRUE;
+    } else if(relitem_is_in_relation(context->item, relation) && 
+	      !gtk_tree_selection_iter_is_selected(sel, &iter)) {
+
+      printf("deselected: "ITEM_ID_FORMAT"\n", OSM_ID(relation));
+
+      relation_remove_item(relation, context->item);
+      gtk_list_store_set(context->store, &iter, 
+		       RELITEM_COL_ROLE, NULL, 
+		       -1);
+
+      done = TRUE;
+    }
+
+    if(!done)
+      ok = gtk_tree_model_iter_next(GTK_TREE_MODEL(context->store), &iter);
+  }
+}
+
+#ifndef FREMANTLE
+/* we handle these events on our own in order to implement */
+/* a very direct selection mechanism (multiple selections usually */
+/* require the control key to be pressed). This interferes with */
+/* fremantle finger scrolling, but fortunately the fremantle */
+/* default behaviour already is what we want. */
+static gboolean on_view_clicked(GtkWidget *widget, GdkEventButton *event,
+				gpointer user_data) {
+  if(event->window == gtk_tree_view_get_bin_window(GTK_TREE_VIEW(widget))) {  
+    GtkTreePath *path;
+    
+    if(gtk_tree_view_get_path_at_pos(GTK_TREE_VIEW(widget), 
+		     event->x, event->y, &path, NULL, NULL, NULL)) {
+      GtkTreeSelection *sel = 
+	gtk_tree_view_get_selection(GTK_TREE_VIEW(widget));
+
+      if(!gtk_tree_selection_path_is_selected(sel, path))
+	gtk_tree_selection_select_path(sel, path);
+      else
+	gtk_tree_selection_unselect_path(sel, path);
+    }
+    return TRUE;
+  }
+  return FALSE;
+}
+#endif
+
+static GtkWidget *relation_item_list_widget(relitem_context_t *context) {
+#ifndef FREMANTLE
+  context->view = gtk_tree_view_new();
+#else
+  context->view = hildon_gtk_tree_view_new(HILDON_UI_MODE_EDIT);
+#endif
+
+#ifdef USE_HILDON
+  /* hildon hides these by default */
+  gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(context->view), TRUE); 
+#endif
+
+  /* change list mode to "multiple" */
+  GtkTreeSelection *selection = 
+    gtk_tree_view_get_selection(GTK_TREE_VIEW(context->view));
+  gtk_tree_selection_set_mode(selection, GTK_SELECTION_MULTIPLE);
+
+#ifndef FREMANTLE
+  /* catch views button-press event for our custom handling */
+  g_signal_connect(context->view, "button-press-event", 
+		   G_CALLBACK(on_view_clicked), context); 
+#endif
+
+  /* --- "Name" column --- */
+  GtkCellRenderer *renderer = gtk_cell_renderer_text_new();
+  g_object_set(renderer, "ellipsize", PANGO_ELLIPSIZE_END, NULL );
+  GtkTreeViewColumn *column = gtk_tree_view_column_new_with_attributes(
+       "Name", renderer, "text", RELITEM_COL_NAME, NULL);
+  gtk_tree_view_column_set_expand(column, TRUE);
+  gtk_tree_view_insert_column(GTK_TREE_VIEW(context->view), column, -1);
+
+  /* --- "Type" column --- */
+  renderer = gtk_cell_renderer_text_new();
+  gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(context->view), 
+      -1, "Type", renderer, "text", RELITEM_COL_TYPE, NULL);
+
+  /* --- "Role" column --- */
+  renderer = gtk_cell_renderer_text_new();
+  gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(context->view), 
+      -1, "Role", renderer, "text", RELITEM_COL_ROLE, NULL);
 
   /* build and fill the store */
   context->store = gtk_list_store_new(RELITEM_NUM_COLS, 
-		G_TYPE_BOOLEAN, G_TYPE_STRING, G_TYPE_STRING, 
+		G_TYPE_STRING, G_TYPE_STRING, 
 	        G_TYPE_STRING, G_TYPE_POINTER);
 
-  list_set_store(context->list, context->store);
+  gtk_tree_view_set_model(GTK_TREE_VIEW(context->view), 
+			  GTK_TREE_MODEL(context->store));
+  g_object_unref(context->store);
 
   // Debatable whether to sort by the "selected" or the "Name" column by
   // default. Both are be useful, in different ways.
   gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(context->store),
                                        RELITEM_COL_NAME, GTK_SORT_ASCENDING);
 
+  /* build a list of iters of all items that should be selected */
+
   GtkTreeIter iter;
   relation_t *relation = context->appdata->osm->relation;
   while(relation) {
@@ -434,28 +400,46 @@ static GtkWidget *relation_item_list_widget(relitem_context_t *context) {
     /* Append a row and fill in some data */
     gtk_list_store_append(context->store, &iter);
     gtk_list_store_set(context->store, &iter,
-       RELITEM_COL_SELECTED, relitem_is_in_relation(context->item, relation),
        RELITEM_COL_TYPE, osm_tag_get_by_key(OSM_TAG(relation), "type"),
        RELITEM_COL_ROLE, relitem_get_role_in_relation(context->item, relation),
        RELITEM_COL_NAME, name,
        RELITEM_COL_DATA, relation,
        -1);
-    
+
+    /* select all relations the current object is part of */
+    if(relitem_is_in_relation(context->item, relation)) 
+      gtk_tree_selection_select_iter(selection, &iter);
+   
     relation = relation->next;
   }
   
-  g_object_unref(context->store);
+  g_signal_connect(G_OBJECT(selection), "changed", 
+		   G_CALLBACK(changed), context);
 
-  list_set_static_buttons(context->list, LIST_BTN_BIG, 
-    G_CALLBACK(on_relation_item_add), G_CALLBACK(on_relation_item_edit),
-    G_CALLBACK(on_relation_item_remove), context);
-
-  relation_item_list_selected(context, FALSE);
-
-  return context->list;
+#ifndef FREMANTLE_PANNABLE_AREA
+  /* put view into a scrolled window */
+  GtkWidget *scrolled_window = gtk_scrolled_window_new(NULL, NULL);
+  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window), 
+				 GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
+  gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrolled_window), 
+				      GTK_SHADOW_ETCHED_IN);
+  gtk_container_add(GTK_CONTAINER(scrolled_window), context->view);
+  return scrolled_window;
+#else
+  /* put view into a pannable area */
+  GtkWidget *pannable_area = hildon_pannable_area_new();
+  gtk_container_add(GTK_CONTAINER(pannable_area), context->view);
+  return pannable_area;
+#endif
 }
 
-void relation_add_dialog(GtkWidget *parent, 
+static void 
+on_dialog_destroy(GtkWidget *widget, gpointer user_data ) {
+  relitem_context_t *context = (relitem_context_t*)user_data;
+  g_free(context);
+}
+
+void relation_membership_dialog(GtkWidget *parent, 
 			 appdata_t *appdata, object_t *object) {
   relitem_context_t *context = g_new0(relitem_context_t, 1);
   map_t *map = appdata->map;
@@ -467,11 +451,11 @@ void relation_add_dialog(GtkWidget *parent,
   char *str = NULL;
   switch(object->type) {
   case NODE:
-    str = g_strdup_printf(_("Relations for node #" ITEM_ID_FORMAT), 
+    str = g_strdup_printf(_("Relation memberships of node #" ITEM_ID_FORMAT), 
 			  OBJECT_ID(*object));
     break;
   case WAY:
-    str = g_strdup_printf(_("Relations for way #" ITEM_ID_FORMAT), 
+    str = g_strdup_printf(_("Relation memberships of way #" ITEM_ID_FORMAT), 
 			  OBJECT_ID(*object));
     break;
   default:
@@ -486,6 +470,9 @@ void relation_add_dialog(GtkWidget *parent,
 		    NULL);
   g_free(str);
   
+  gtk_signal_connect(GTK_OBJECT(context->dialog), "destroy", 
+	     (GtkSignalFunc)on_dialog_destroy, context);
+
   gtk_dialog_set_default_response(GTK_DIALOG(context->dialog), 
 				  GTK_RESPONSE_CLOSE);
 
@@ -497,8 +484,6 @@ void relation_add_dialog(GtkWidget *parent,
   gtk_widget_show_all(context->dialog);
   gtk_dialog_run(GTK_DIALOG(context->dialog));
   gtk_widget_destroy(context->dialog);
-
-  g_free(context);
 }
 
 /* -------------------- global relation list ----------------- */
@@ -511,8 +496,7 @@ typedef struct {
 } relation_context_t;
 
 enum {
-  RELATION_COL_ID = 0,
-  RELATION_COL_TYPE,
+  RELATION_COL_TYPE = 0,
   RELATION_COL_NAME,
   RELATION_COL_MEMBERS,
   RELATION_COL_DATA,
@@ -545,22 +529,17 @@ static void relation_list_selected(relation_context_t *context,
   list_button_enable(context->list, LIST_BUTTON_EDIT, selected != NULL);
 }
 
-static gboolean
-relation_list_selection_func(GtkTreeSelection *selection, GtkTreeModel *model,
-		     GtkTreePath *path, gboolean path_currently_selected,
-		     gpointer userdata) {
+static void
+relation_list_changed(GtkTreeSelection *selection, gpointer userdata) {
   relation_context_t *context = (relation_context_t*)userdata;
+  GtkTreeModel *model = NULL;
   GtkTreeIter iter;
-    
-  if(gtk_tree_model_get_iter(model, &iter, path)) {
-    g_assert(gtk_tree_path_get_depth(path) == 1);
-    
+  
+  if(gtk_tree_selection_get_selected(selection, &model, &iter)) {
     relation_t *relation = NULL;
     gtk_tree_model_get(model, &iter, RELATION_COL_DATA, &relation, -1);
     relation_list_selected(context, relation);
   }
-  
-  return TRUE; /* allow selection state to change */
 }
 
 typedef struct {
@@ -600,7 +579,12 @@ member_list_selection_func(GtkTreeSelection *selection, GtkTreeModel *model,
 
 static GtkWidget *member_list_widget(member_context_t *context) {
   GtkWidget *vbox = gtk_vbox_new(FALSE,3);
+
+#ifndef FREMANTLE_PANNABLE_AREA
   context->view = gtk_tree_view_new();
+#else
+  context->view = hildon_gtk_tree_view_new(HILDON_UI_MODE_EDIT);
+#endif
 
   gtk_tree_selection_set_select_function(
 	 gtk_tree_view_get_selection(GTK_TREE_VIEW(context->view)), 
@@ -684,6 +668,7 @@ static GtkWidget *member_list_widget(member_context_t *context) {
 
   g_object_unref(context->store);
 
+#ifndef FREMANTLE_PANNABLE_AREA
   /* put it into a scrolled window */
   GtkWidget *scrolled_window = gtk_scrolled_window_new(NULL, NULL);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window), 
@@ -693,6 +678,12 @@ static GtkWidget *member_list_widget(member_context_t *context) {
   gtk_container_add(GTK_CONTAINER(scrolled_window), context->view);
 
   gtk_box_pack_start_defaults(GTK_BOX(vbox), scrolled_window);
+#else
+  /* put view into a pannable area */
+  GtkWidget *pannable_area = hildon_pannable_area_new();
+  gtk_container_add(GTK_CONTAINER(pannable_area), context->view);
+  gtk_box_pack_start_defaults(GTK_BOX(vbox), pannable_area);
+#endif
 
   return vbox;
 }
@@ -777,7 +768,6 @@ static void on_relation_add(GtkWidget *but, relation_context_t *context) {
     GtkTreeIter iter;
     gtk_list_store_append(context->store, &iter);
     gtk_list_store_set(context->store, &iter,
-		       RELATION_COL_ID, OSM_ID(relation),
 		       RELATION_COL_TYPE,
 		       osm_tag_get_by_key(OSM_TAG(relation), "type"),
 		       RELATION_COL_NAME, name,
@@ -786,10 +776,6 @@ static void on_relation_add(GtkWidget *but, relation_context_t *context) {
 		       -1);
 
     gtk_tree_selection_select_iter(list_get_selection(context->list), &iter);
-
-    /* scroll to end */
-    //    GtkAdjustment *adj = gtk_scrolled_window_get_vadjustment();
-    /* xyz */
   }
 }
 
@@ -821,7 +807,6 @@ static void on_relation_edit(GtkWidget *but, relation_context_t *context) {
 
   // Found it. Update all visible fields.
   gtk_list_store_set(context->store, &iter,
-    RELATION_COL_ID,      OSM_ID(sel),
     RELATION_COL_TYPE,    osm_tag_get_by_key(OSM_TAG(sel), "type"),
     RELATION_COL_NAME,    relation_get_descriptive_name(sel),
     RELATION_COL_MEMBERS, osm_relation_members_num(sel),
@@ -850,8 +835,8 @@ static void on_relation_remove(GtkWidget *but, relation_context_t *context) {
   
   /* first remove selected row from list */
   GtkTreeIter       iter;
-  GtkTreeSelection *selection = list_get_selection(context->list);
-  if(gtk_tree_selection_get_selected(selection, NULL, &iter)) 
+  GtkTreeModel      *model;
+  if(list_get_selected(context->list, &model, &iter)) 
     gtk_list_store_remove(context->store, &iter);
 
   /* then really delete it */
@@ -863,19 +848,17 @@ static void on_relation_remove(GtkWidget *but, relation_context_t *context) {
 static GtkWidget *relation_list_widget(relation_context_t *context) {
   context->list = list_new(LIST_HILDON_WITH_HEADERS);
 
-  list_set_selection_function(context->list, relation_list_selection_func, 
-			      context);
+  list_override_changed_event(context->list, relation_list_changed, context);
 
   list_set_columns(context->list,
-		   _("Id"),      RELATION_COL_ID, 0,
-		   _("Type"),    RELATION_COL_TYPE, 0,
 		   _("Name"),    RELATION_COL_NAME, LIST_FLAG_ELLIPSIZE,
+		   _("Type"),    RELATION_COL_TYPE, 0,
 		   _("Members"), RELATION_COL_MEMBERS, 0,
 		   NULL);
 
   /* build and fill the store */
   context->store = gtk_list_store_new(RELATION_NUM_COLS, 
-		G_TYPE_ITEM_ID_T, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_UINT, 
+		G_TYPE_STRING, G_TYPE_STRING, G_TYPE_UINT, 
 	        G_TYPE_POINTER);
 
   list_set_store(context->list, context->store);
@@ -903,7 +886,6 @@ static GtkWidget *relation_list_widget(relation_context_t *context) {
     /* Append a row and fill in some data */
     gtk_list_store_append(context->store, &iter);
     gtk_list_store_set(context->store, &iter,
-		       RELATION_COL_ID, OSM_ID(rel),
 		       RELATION_COL_TYPE,
 		       osm_tag_get_by_key(OSM_TAG(rel), "type"),
 		       RELATION_COL_NAME, name,
@@ -920,7 +902,7 @@ static GtkWidget *relation_list_widget(relation_context_t *context) {
   
   g_object_unref(context->store);
 
-  list_set_static_buttons(context->list, LIST_BTN_NEW, 
+  list_set_static_buttons(context->list, LIST_BTN_NEW | LIST_BTN_WIDE, 
 	  G_CALLBACK(on_relation_add), G_CALLBACK(on_relation_edit), 
 	  G_CALLBACK(on_relation_remove), context);
 
